@@ -5,7 +5,6 @@
 #include "log.h"
 
 #include <jansson.h>
-#include <openssl/sha.h>
 
 #include "utils.h"
 #include "db.h"
@@ -38,23 +37,25 @@ commit_from_json_object (const char *id, json_t *object);
 
 static void compute_commit_id (SeafCommit* commit)
 {
-    SHA_CTX ctx;
+    GChecksum *ctx = g_checksum_new(G_CHECKSUM_SHA1);
     uint8_t sha1[20];    
     gint64 ctime_n;
 
-    SHA1_Init (&ctx);
-    SHA1_Update (&ctx, commit->root_id, 41);
-    SHA1_Update (&ctx, commit->creator_id, 41);
+    g_checksum_update (ctx, (guchar *)commit->root_id, 41);
+    g_checksum_update (ctx, (guchar *)commit->creator_id, 41);
     if (commit->creator_name)
-        SHA1_Update (&ctx, commit->creator_name, strlen(commit->creator_name)+1);
-    SHA1_Update (&ctx, commit->desc, strlen(commit->desc)+1);
+        g_checksum_update (ctx, (guchar *)commit->creator_name, strlen(commit->creator_name)+1);
+    g_checksum_update (ctx, (guchar *)commit->desc, strlen(commit->desc)+1);
 
     /* convert to network byte order */
     ctime_n = hton64 (commit->ctime);
-    SHA1_Update (&ctx, &ctime_n, sizeof(ctime_n));
-    SHA1_Final (sha1, &ctx);
+    g_checksum_update (ctx, (guchar *)&ctime_n, sizeof(ctime_n));
+
+    gsize len = 20;
+    g_checksum_get_digest (ctx, sha1, &len);
     
     rawdata_to_hex (sha1, commit->commit_id, 20);
+    g_checksum_free (ctx);
 }
 
 SeafCommit*
@@ -164,6 +165,9 @@ seaf_commit_free (SeafCommit *commit)
     g_free (commit->client_version);
     g_free (commit->magic);
     g_free (commit->random_key);
+    g_free (commit->pwd_hash);
+    g_free (commit->pwd_hash_algo);
+    g_free (commit->pwd_hash_params);
     g_free (commit);
 }
 
@@ -588,12 +592,17 @@ commit_to_json_object (SeafCommit *commit)
 
     if (commit->encrypted) {
         json_object_set_int_member (object, "enc_version", commit->enc_version);
-        if (commit->enc_version >= 1)
+        if (commit->enc_version >= 1 && !commit->pwd_hash)
             json_object_set_string_member (object, "magic", commit->magic);
         if (commit->enc_version >= 2)
             json_object_set_string_member (object, "key", commit->random_key);
         if (commit->enc_version >= 3)
             json_object_set_string_member (object, "salt", commit->salt);
+        if (commit->pwd_hash) {
+            json_object_set_string_member (object, "pwd_hash", commit->pwd_hash);
+            json_object_set_string_member (object, "pwd_hash_algo", commit->pwd_hash_algo);
+            json_object_set_string_member (object, "pwd_hash_params", commit->pwd_hash_params);
+        }
     }
     if (commit->version != 0)
         json_object_set_int_member (object, "version", commit->version);
@@ -628,6 +637,9 @@ commit_from_json_object (const char *commit_id, json_t *object)
     const char *magic = NULL;
     const char *random_key = NULL;
     const char *salt = NULL;
+    const char *pwd_hash = NULL;
+    const char *pwd_hash_algo = NULL;
+    const char *pwd_hash_params = NULL;
     int version = 0;
     int conflict = 0, new_merge = 0;
     int repaired = 0;
@@ -661,6 +673,9 @@ commit_from_json_object (const char *commit_id, json_t *object)
         && json_object_has_member (object, "enc_version")) {
         enc_version = json_object_get_int_member (object, "enc_version");
         magic = json_object_get_string_member (object, "magic");
+        pwd_hash = json_object_get_string_member (object, "pwd_hash");
+        pwd_hash_algo = json_object_get_string_member (object, "pwd_hash_algo");
+        pwd_hash_params = json_object_get_string_member (object, "pwd_hash_params");
     }
 
     if (enc_version >= 2)
@@ -688,6 +703,10 @@ commit_from_json_object (const char *commit_id, json_t *object)
         (second_parent_id && !is_object_id_valid(second_parent_id)))
         return commit;
 
+    // If pwd_hash is set, the magic field is no longer included in the commit of the newly created repo.
+    if (!magic)
+        magic = pwd_hash;
+
     switch (enc_version) {
     case 0:
         break;
@@ -743,12 +762,17 @@ commit_from_json_object (const char *commit_id, json_t *object)
 
     if (commit->encrypted) {
         commit->enc_version = enc_version;
-        if (enc_version >= 1)
+        if (enc_version >= 1 && !pwd_hash)
             commit->magic = g_strdup(magic);
         if (enc_version >= 2)
             commit->random_key = g_strdup (random_key);
         if (enc_version >= 3)
             commit->salt = g_strdup (salt);
+        if (pwd_hash) {
+            commit->pwd_hash = g_strdup (pwd_hash);
+            commit->pwd_hash_algo = g_strdup (pwd_hash_algo);
+            commit->pwd_hash_params = g_strdup (pwd_hash_params);
+        }
     }
     commit->version = version;
     if (new_merge)
